package dev.fyloz.memorycache.utils

/**
 * Removes the first element matching the given [predicate] in this [MutableIterable].
 * Returns true if any element was removed.
 */
inline fun <T> MutableIterable<T>.removeFirst(predicate: (T) -> Boolean): Boolean {
    with(iterator()) {
        while (hasNext()) {
            if (predicate(next())) {
                remove()
                return true
            }
        }
    }

    return false
}

/** Gets the nth item in this [Iterable]. */
inline fun <T> Iterable<T>.nth(n: Int, predicate: (T) -> Boolean): T? {
    var matches = 0

    for (item in this) {
        if (!predicate(item)) {
            continue
        }

        matches++

        if (matches == n) {
            return item
        }
    }

    return null
}

/** Is this [Int] is even or not. */
val Int.isEven: Boolean
    get() = this % 2 == 0
